#include "tdsutil.h"
#include "tds.h"
#include "sybfront.h"
#include "sybdb.h"
#include "dblib.h"
#include <unistd.h>

int _get_conversion_type(int srctype, int colsize)
{
	if (srctype == SYBINTN) {
		if (colsize==4)
			 return SYBINT4;
		else if (colsize==2)
			 return SYBINT2;
		else if (colsize==1)
			 return SYBINT1;
	} else if (srctype == SYBFLTN) {
		if (colsize==8)
			return SYBFLT8;
		else if (colsize==4)
			return SYBREAL;
	}
	return srctype;
}
DBINT _convert_char(int srctype,BYTE *src,
	int desttype,BYTE *dest,DBINT destlen)
{
	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			if (strlen(src)>destlen) {
				strncpy(dest,src,destlen-1);
				dest[destlen-1]='\0';
				return destlen;
			} else {
				strcpy(dest, src);
				return (strlen(dest));
			}
			break;
		case SYBINTN:
			break;
	}
	return FAIL;
}
DBINT _convert_bit(int srctype,BYTE *src,
	int desttype,BYTE *dest,DBINT destlen)
{
DBTINYINT srcval;
DBSMALLINT destval2;
DBINT destval4;

	memcpy(&srcval,src,1);
	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			sprintf(dest,"%c",srcval ? '1' : '0');
			break;
		case SYBINT1:
			memcpy(dest,src,1);
			break;
		case SYBINT2:
			memcpy(dest,&destval2,2);
			break;
		case SYBINT4:
			destval4=srcval;
			memcpy(dest,&destval4,4);
			break;
	}
}
DBINT _convert_int1(int srctype,BYTE *src,
	int desttype,BYTE *dest,DBINT destlen)
{
DBTINYINT srcval;
DBSMALLINT destval2;
DBINT destval4;
	
	memcpy(&srcval,src,1);
	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			sprintf(dest,"%ld",srcval);
			break;
		case SYBINT1:
			memcpy(dest,src,1);
			break;
		case SYBINT2:
			memcpy(dest,&destval2,2);
			break;
		case SYBINT4:
			destval4=srcval;
			memcpy(dest,&destval4,4);
			break;
	}
}
DBINT _convert_int2(int srctype,BYTE *src,
	int desttype,BYTE *dest,DBINT destlen)
{
DBSMALLINT srcval;
DBINT destval4;
	
	memcpy(&srcval,src,2);
	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			sprintf(dest,"%ld",srcval);
			break;
		case SYBINT2:
			memcpy(dest,&srcval,2);
			break;
		case SYBINT4:
			destval4=srcval;
			memcpy(dest,&destval4,4);
			break;
	}

}
DBINT _convert_int4(int srctype,BYTE *src,
	int desttype,BYTE *dest,DBINT destlen)
{
DBINT srcval;
	
	memcpy(&srcval,src,4);

	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			sprintf(dest,"%ld",srcval);
			break;
		case SYBINT4:
			memcpy(dest,&srcval,4);
			break;
	}
	return FAIL;
}
DBINT _convert_money(int srctype,BYTE *src,
	int desttype,BYTE *dest,DBINT destlen)
{
unsigned long dollars, hundreths,bignum;

	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			/* FIX ME -- this will not handle really big money */
			bignum=(unsigned char) src[4]+
				(unsigned char) src[5]*256+
				(unsigned char) src[6]*256*256+
				(unsigned char) src[7]*256*256*256;
			dollars=bignum/10000;
			hundreths=bignum%10000;
			sprintf(dest,"%lu.%02lu",dollars,hundreths/100);
			break;
	}
}

DBINT _convert_datetime(int srctype,BYTE *src,int desttype,BYTE *dest,DBINT destlen)
{
DBDATETIME* in = (DBDATETIME*)src;
long tmp_secs_from_epoch;

	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			tmp_secs_from_epoch = ((in->dtdays - 25567)*(24*60*60))
                        	+(in->dttime/300);
			if (strlen(src)>destlen) {
				strftime(dest, destlen-1, "%b %d %Y %I:%M%p",
				(struct tm*)gmtime(&tmp_secs_from_epoch));
				return destlen;
			} else {
				strftime(dest, 20, "%b %d %Y %I:%M%p",
				(struct tm*)gmtime(&tmp_secs_from_epoch));
				return (strlen(dest));
			}
			break;
	}
}
DBINT _convert_flt8(int srctype,BYTE *src,int desttype,BYTE *dest,DBINT destlen)
{
DBFLT8 the_value = *(double*)src;

	switch(desttype) {
		case SYBCHAR:
		case SYBVARCHAR:
			sprintf(dest,"%10.0lf", the_value);
	}
}

